@@ -11,16 +11,19 @@ server <- function(input, output, session) {
   sendRecentProjectsToGui();
   sendInformationAboutBackendToGui();
 
-  shiny::observeEvent(input$triggerFetchDataset, {
-    downloadDataset(input);
-  })
-
-  shiny::observeEvent(input$verifyColumnsNames, {
-    verifyColumnsNames(input$dataFrameToVerify, input$properColNames)
+  shiny::observeEvent(input$triggerImport, {
+    importDataset(input$baseUrl, input$projectId, input$identityToken,
+                  input$datasetId, input$originalName, input$saveAsName, input$datasetType,
+                  input$datasetBody, input$instanceId);
   })
 
   shiny::observeEvent(input$triggerExportDataframes, {
-    exportDataset(input);
+    exportDataframes(input$baseUrl, input$projectId, input$identityToken, input$saveAsName,
+                    input$description, input$selectedDataframes, input$folderId, input$certify)
+  })
+
+    shiny::observeEvent(input$triggerCubeUpdate, {
+    updateCube(input$baseUrl, input$projectId, input$identityToken, input$cubeId, input$cubeName, input$updatePolicies)
   })
 
   shiny::observeEvent(input$onExportDataMode,{
@@ -43,11 +46,7 @@ server <- function(input, output, session) {
     applyDataModeling(input$dataModelingSteps, input$selectedObjects)
   })
 
-  shiny::observeEvent(input$triggerCubeUpdate, {
-    updateCube(input)
-  })
-
-  shiny::observeEvent(input$generatedCode, {
+  shiny::observeEvent(input$triggerGenerateCode, {
     displayGeneratedCode(input$generatedCode)
   })
 
